<?php

class Logger
{
    private string $logFile;

    public function __construct()
    {
        $this->logFile = __DIR__ . '/log.txt';
        set_error_handler([$this, 'logError']);
        set_exception_handler([$this, 'logException']);
    }

    public function __destruct()
    {
        restore_error_handler();
        restore_exception_handler();
    }

    public function logError(int $error_number, string $error_string, string $error_file, int $error_line)
    {
        $errorLevel = $this->getErrorLevel($error_number);
        $date = new DateTime();

        $message = $this->formatMessage('Error', $errorLevel, $error_string, $error_file, $error_line, $date);
        $this->writeLog($message);
    }

    public function logException(Throwable $exception)
    {
        $date = new DateTime();
        $message = $this->formatMessage('Exception', $exception->getCode(), $exception->getMessage(), $exception->getFile(), $exception->getLine(), $date);
        $this->writeLog($message);
    }

    private function getErrorLevel(int $error_number): string
    {
        switch ($error_number) {
            case E_NOTICE:
            case E_USER_NOTICE:
                return 'Notice';
            case E_WARNING:
            case E_USER_WARNING:
                return 'Warning';
            case E_ERROR:
            case E_USER_ERROR:
                return 'Fatal Error';
            default:
                return 'Unknown Error';
        }
    }

    public function formatMessage(string $type, $level, string $message, string $file, int $line, DateTime $date)
    {
        $log = "------------------------------------\n" .
        "Type : $type\n" .
        "Level : $level\n" .
        "Message : $message\n" .
        "File : $file\n" .
        "Line : $line\n" .
        "Date : " . $date->format('Y-m-d H:i:s') . "\n";
        
        return $log;
    }

    public function writeLog(string $message)
    {
        file_put_contents($this->logFile, $message, FILE_APPEND);
    }
}

$logger = new Logger();
